//Externes EEProm ber I2C

#include <stdio.h>
#include "lcd.h"
#include <util/delay.h>
#include <i2cmaster.h>
#define Dev24C02  0xA0	

//IC eeprom:
void init_eeprom(void) {
	i2c_init();                                // init I2C interface
}


//Low-Level eeprom-Zugriff
static void eeprom_write (uint16_t address, uint8_t value) {
   unsigned char ret=0;
	
	uint8_t address_msb = address / 256;
	uint8_t address_lsb = address - (256*address_msb);
	
	//_delay_ms (1);
	
	lcd_gotoxy (0,0); 
	char string[41];
	sprintf (string, "EEPROM-write-start");
	lcd_puts(string);
		
	
	
    ret = i2c_start_wait(Dev24C02+I2C_WRITE);       // set device address and write mode
    //1010xxxy
	if ( ret ) {
       // failed to issue start condition, possibly no device found 
        i2c_stop();

		lcd_gotoxy (0,0); 
		char string[41];
		sprintf (string, "EEPROM-FEHLER: %d %d %d  %d", Dev24C02, I2C_WRITE, address_msb, address);
		lcd_puts(string);
		for (;;);
	//}	
    } else {
         //issuing start condition ok, device accessible 
        i2c_write(address_msb);                     // write address = 5
		i2c_write(address_lsb);                     // write address = 5
        i2c_write(value);                      // ret=0 -> Ok, ret=1 -> no ACK 
        i2c_stop();                            // set stop conditon = release bus
	}
	//_delay_ms(1000);
}

static uint8_t eeprom_read (uint16_t address) {	
	unsigned char ret;
	
	uint8_t address_msb = address / 256;
	uint8_t address_lsb = address - (256*address_msb);
	
	//read value back from eeprom address, 
	//wait until the device is no longer busy from the previous write operation 
	//1010xxxy   (x: device address=0. y:r/w)
    i2c_start_wait(Dev24C02+I2C_WRITE);     // set device address and write mode
	//|(address_msb<<1)
    i2c_write(address_msb);                     // write address = 5
	i2c_write(address_lsb);                     // write address = 5
	
    i2c_rep_start(Dev24C02+I2C_READ);       // set device address and read mode
    
	ret = i2c_readNak();                    // read one byte
    i2c_stop();
	return ret;
}

/*Konstanten fr eeprom-Zugriff (Anforderung oder Speicherung von Daten
#define _MODE 0
#define _FUNCTION 1
#define _FVALUE 2
#define _ANVALUE 3
#define _AUSVALUE 4
#define _CHANNEL 5

Eeprom-Belegung fr externes eeprom: Fr jede Eigenschaft 1 Byte.
Mode: 		0-3	    8 Bit
Function: 	0-3	    8 Bit
Channel:	0-15	8 Bit
Fvalue:		0-127	8 Bit
Anvalue:	0-129	8 Bit
Ausvalue:	0-129	8 Bit
=> 1 Byte pro Einstellung, 6 Bytes pro Steuerelement ==> 288 Bytes fr 48 Steuerelemente ==> 7 Profile mglich
*/

uint8_t exteeprom_lesen (uint8_t profil, uint8_t steuerelement, uint8_t einstellung) {
	return eeprom_read(288*profil + 6*steuerelement + 1*einstellung);
}


void exteeprom_schreiben (uint8_t profil, uint8_t steuerelement, uint8_t einstellung, uint8_t wert) {
	if (exteeprom_lesen(profil, steuerelement, einstellung) != wert) { //Nur bei Differenz schreiben.
		eeprom_write(288*profil + 6*steuerelement + 1*einstellung, wert);
	}
}
